#include <pic.h>

	__CONFIG(INTIO & WDTEN & UNPROTECT & BOREN & MCLRDIS)

#define LCD

#define SHIFT_ENABLED  0

#define FRAMES_COUNT

//#define SPEED_LIMITING
//#define FILTERING
#define COARSE_READINGS

#include "types.h"

//--- PORT DEFINITIONS ---------------------------------------------------
//               76543210
#define  TrisA 0b00011111                
//               ||||||||
//               |||||||GYRO_VERT
//               ||||||VREF3300
//               |||||SERVO_IN
//               ||||BUTTON
//               |||GYRO_HORIZ
//               ||SERVO_OUT
//               |---
//               ---

#ifdef LCD
//               76543210
#define  TrisC 0b00000000
//               ||||||||
//               |||||||D4
//               ||||||D5
//               |||||D6 
//               ||||D7
//               |||E 
//               ||A0
//               |---
//               ---
#else
//               76543210
#define  TrisC 0b00111111
//               ||||||||
//               |||||||TRIM_H
//               ||||||EXT_V
//               |||||EXT_H
//               ||||TRIM_V
//               |||VERT_REV
//               ||HORIZ_REV
//               |---
//               ---
#endif

#define	P_GYRO_VERT	RA0   //        - PIC-13 (AN0)
#define	VREF3300  	RA1   //        - PIC-12 (AN1, VRef+)
#define	SERVO_IN  	RA2   //        - PIC-11 (AN2, pin of INTerrupt)
#define BUTTON	  	RA3   //        - PIC-4
#define P_GYRO_HORIZ	RA4   //        - PIC-3  (AN3)
#define	SERVO_OUT 	RA5   //        - PIC-2 

#ifdef LCD
#define	LCD_A0    RC5   // LCD-4  - PIC-5 
#define	LCD_E     RC4   // LCD-6  - PIC-6 
#define	LCD_D4    RC0   // LCD-11 - PIC-10
#define	LCD_D5    RC1   // LCD-12 - PIC-9
#define	LCD_D6    RC2   // LCD-13 - PIC-8
#define	LCD_D7    RC3   // LCD-14 - PIC-7
#else
#define	TRIM_V    RC3   //        - PIC-7  (AN7)
#define	TRIM_H    RC0   //        - PIC-10 (AN4)
#define	EXT_V     RC1   //        - PIC-9  (AN5)
#define	EXT_H     RC2   //        - PIC-8  (AN6)
#define	VERT_REV  RC4   //        - PIC-6
#define	HORIZ_REV RC5   //        - PIC-5
#endif

//--- ANALOG SETUP DATA --------------------------------------------------
#define ADChannel_GYRO_VERT 	0
#define ADChannel_GYRO_HORIZ	3
#define ADChannel_TrimV 	7
#define ADChannel_TrimH		4
#define ADChannel_ExtV 		5
#define ADChannel_ExtH 		6

#define NUM_ADC_Channels	4

#define iTrimV			0
#define iTrimH			1
#define iExtV			2
#define iExtH			3

static bank1 const byte ADC_Channels[NUM_ADC_Channels] = {
	    ADChannel_TrimV, ADChannel_TrimH,  ADChannel_ExtV, ADChannel_ExtH};
static bank1 uint16 ADC_Data[NUM_ADC_Channels];

#define GYRO_ADC_SAMPLES	8

//--------------------------------------------------------------------------------------

// Maximum input speed to eliminate jitter;
#define MAX_VERT_SERVO_SPEED  10
#define MAX_HORIZ_SERVO_SPEED 10

// Average points;
#define NUM_AVG_SAMPLES 8      // 8, 16, 32=0.5 sec delay

//--------------------------------------------------------------------------------------
// ISR variables;

#define NUM_SERVO 3

// Servo[0] = Horizontal pan; Servo[1]= Vertical tilt; Servo[2]=AP shutter sw.;
static uint16 Servo[NUM_SERVO];

//--------------------------------------------------------------------------------------
// Filtering;

static bank1 uint16 ServoData_V[NUM_AVG_SAMPLES-1];
static bank2 uint16 ServoData_H[NUM_AVG_SAMPLES-1];

//--------------------------------------------------------------------------------------

static byte CurrentServo;
static bit  ServoStart;
static bit  WaitForGap;

#ifdef FRAMES_COUNT
static uint16 FramesCount;
static uint16 BrokenFramesCount;
#endif

static bit  Shift; // shift servo data to one channel (i.e. 4 -> 5);
static bit  Vert_Rev; 
static bit  Horiz_Rev; 

static byte _StartCh; // calculates at the begginning of frame;

static uint16 v, h;

//--------------------------------------------------------------------------------------
// Servo related stuff;

#define SHIFT_NEUTRAL   -250
#define SERVO_PERIOD	20000
#define SERVO_NEUTRAL	1500
#define SERVO_SPAN	500
#define SERVO_MAX (SERVO_NEUTRAL+SERVO_SPAN)
#define SERVO_MIN (SERVO_NEUTRAL-SERVO_SPAN)
#define SERVO_START	300
#define SERVO_GAP	(3*SERVO_MAX)

#define START_CH	 4 // Servo[0] = Rudder(Ch.4); Servo[1]= Switch(Ch.5); Servo[2]=(Ch.6);
#define START_CH_SHIFTED 6 // Servo[0] = 6; Servo[1]= 7; Servo[2]=8;

#define SERVO_HIGH	0 // 0=inverse servo polarity (Hitec Flash 5 needs 0);

//--------------------------------------------------------------------------------------

#define BUTTON_PRESSED 0
#define REVERSE_ENABLED 0

#ifdef LCD
#include "test.c"
#endif

//--------------------------------------------------------------------------------------
// Gyro support;

#define GYRO_SENSETIVITY	2.0	// 1/(mV/deg/S)
#define VREF_VOLTAGE		3300.0	// mV
#define TIME_SCALE_SEC		1e-6
#define CORR			6
#define COEFF 			CORR*((1.0/GYRO_SENSETIVITY*TIME_SCALE_SEC)/1024*VREF_VOLTAGE/GYRO_ADC_SAMPLES)

#define NUM_INTEGRATION_STEPS 4
// Runge-Kutta integration method is uses:
// I(i)=I(i-1)+1/6*(V[i-3]+2*V[i-2]+2*V[i-1]+V[i]);
// V[i-3]=raw_data[3];
// V[i-2]=raw_data[2];
// V[i-1]=raw_data[1];
// V[i]=raw_data[0];

typedef struct {
	int16 raw_data[NUM_INTEGRATION_STEPS];
        uint16 raw_zero;
	double angle;
} GyroChannelStruct;

#define GYRO_VERT	0
#define GYRO_HORIZ	1

static bank1 GyroChannelStruct GyroChannel[2];

static bit resetGyro;

static const double MAX_ANGLE[] = {90.0, 90.0};

//--------------------------------------------------------------------------------------

#include "delay8m.c"

#include "lcd8m.c"

static bit  button;  

static uint16 delayCounter = 0;
static uint16 TMR1_Value = 0;
static byte numIncrements = 0;
static bit WaitForEdge = 0;

//--------------------------------------------------------------------------------------

void StartWaitForGap(void);
void StartWaitForGapEx(void);

static void interrupt
isr(void)			// Here be interrupt function - the name is unimportant.
{
  // handle the interrupt (insert application code here)

// INTE enabled;
// INTEDG = 1 (0->1);
// needs 3-4 cycles to get into interrupt;

   if (INTF == 1) { // --------- SERVO_EDGE_Interrupt 

     WaitForEdge = 0;

     if (WaitForGap) { // found impulse during waiting gap -> need to restart WaitForGap;

#ifdef FRAMES_COUNT
       BrokenFramesCount++;
#endif
       StartWaitForGapEx();

     } else {

       if (CurrentServo == 0) {
#ifdef FRAMES_COUNT
          FramesCount++;
#endif
          _StartCh = START_CH;
          if (Shift == 1)
	     _StartCh = START_CH_SHIFTED;
       }

       if (CurrentServo < _StartCh) { // 0, 1, 2, 3, (4)

         // start new synchro;
         SERVO_OUT = SERVO_HIGH;

         T1CON = 0b00000000; // Stop timer;
         {
           uint16 t;
           ((WORD*)(&t))->low = TMR1L; 
           ((WORD*)(&t))->high = TMR1H; 
           t = 65536-t; 
           delayCounter += (TMR1_Value-t)/2;
           numIncrements++;
         }
         TMR1H = ((65536-SERVO_START*2) >> 8) & 0xFF; 
         TMR1L = (65536-SERVO_START*2) & 0xFF; // start counting;
         T1CON = 0b00000001; // Resume counting;
         TMR1_Value = SERVO_START*2;
         CurrentServo++; 
       }

     } 

     INTF = 0;  /* reset flag */

   } // else { // ----------- TMR1_Interrupt 
   
     if (TMR1IF) {

       delayCounter += TMR1_Value/2;
       numIncrements++;

       if (WaitForEdge) {
#ifdef FRAMES_COUNT
         BrokenFramesCount++;
#endif
         StartWaitForGap();
         WaitForEdge = 0;

       } else {

       if (WaitForGap) { // Gap found, because timer finishes count and no INT occurs;
         // Timer1 is inactive - we wait for edge INT;
         WaitForGap = 0;  // now we sit inside gap;
         CurrentServo = 0; // start of sequence;
         WaitForEdge = 1;
       } else {

         T1CON = 0b00000000; // Stop timer;
  
         if (CurrentServo < _StartCh) { 
            SERVO_OUT = !SERVO_HIGH;
         } else {

          if (CurrentServo == NUM_SERVO+_StartCh) { // 2+4=6
            SERVO_OUT = !SERVO_HIGH;

            // we finished short "1" impulse followed by last ch., so we will wait the gap;
            StartWaitForGap();

          } else {

            uint16 width;
            if (ServoStart == 1) {

              SERVO_OUT = SERVO_HIGH;
              width = SERVO_START;
              ServoStart = 0;
              CurrentServo++;

            } else {
              // _Shift does not influence array item position;
              byte ServoPos = CurrentServo-_StartCh;
              SERVO_OUT = !SERVO_HIGH;
              width = Servo[ServoPos]; // 4.. -> 0..
              width -= SERVO_START;

              ServoStart = 1; // followed by short "1" for channel delimiter; 

              if (width == 0)
                width = 1;
            }

            width *= 2;

            TMR1H = ((65536-width) >> 8) & 0xFF; ;
            TMR1L = (65536-width) & 0xFF;

            T1CON = 0b00000001; // Resume counting;

            TMR1_Value = width;
          }

         } 

       }
       }

       TMR1IF = 0;         
     }

//   }

}

//--------------------------------------------------------------------------------------

uint16 ReadADC(byte piADChannel, byte numSamples) {
  byte i;
  uint16 res = 0;

  byte a = (piADChannel << 2);
  ADCON0 = (ADCON0 & 0b11100011) | a;
  NOP(); NOP(); NOP(); NOP(); NOP();
  
  for (i=0; i < numSamples; i++) {
    GODONE = 1;
    while (GODONE) { };
    res += ((((uint16)ADRESH) << 8) | ADRESL);
  }

  return res;
}

//--------------------------------------------------------------------------------------

static uint16 timeElasped;

void getTimeElapsed(void) {
   uint16 t;
   GIE = 0;
   T1CON = 0b00000000; // Stop timer;
   ((WORD*)(&t))->low = TMR1L; 
   ((WORD*)(&t))->high = TMR1H; 
   T1CON = 0b00000001; // Resume counting;
   t = 65536-t; 
   timeElasped = delayCounter + (TMR1_Value-t)/2;
   delayCounter = 0; numIncrements=0;
   TMR1_Value = t;
   GIE = 1;
}

//--------------------------------------------------------------------------------------

static byte curChannel = 0;

void readChannels(void) {
  int ADC_Val;

  GyroChannel[GYRO_VERT].raw_data[0] = ReadADC(ADChannel_GYRO_VERT, GYRO_ADC_SAMPLES);
  GyroChannel[GYRO_HORIZ].raw_data[0] = ReadADC(ADChannel_GYRO_HORIZ, GYRO_ADC_SAMPLES);

  if (curChannel >= NUM_ADC_Channels)
    curChannel = 0;

#ifndef LCD

  ADC_Data[curChannel] = ReadADC(ADC_Channels[curChannel], 1);

  Vert_Rev = (VERT_REV == REVERSE_ENABLED);
  Horiz_Rev = (HORIZ_REV == REVERSE_ENABLED);

#else

  switch (curChannel) {
case iTrimV: 
  ADC_Val = TEST_TrimV; break;
case iTrimH: 
  ADC_Val = TEST_TrimH; break;
case iExtV: 
  ADC_Val = TEST_ExtV; break;
case iExtH: 
  ADC_Val = TEST_ExtH; break;
  }
  ADC_Data[curChannel] = ADC_Val;

  Vert_Rev = (TEST_Vert_Rev == REVERSE_ENABLED);
  Horiz_Rev = (TEST_Horiz_Rev == REVERSE_ENABLED);

#endif

  curChannel++;

  Shift = SHIFT_ENABLED;
  button = BUTTON == BUTTON_PRESSED;
}

//--------------------------------------------------------------------------------------

void initADC(void) {
  byte i;
  byte l;
  for (i=GYRO_VERT; i<=GYRO_HORIZ; i++) {  
  	GyroChannelStruct * p = &(GyroChannel[i]);
	for (l=0; l<=NUM_INTEGRATION_STEPS-1; l++) {
		p->raw_data[l] = 0;
	}
  	p->angle = 0;
	p->raw_zero = 0;
  }
  resetGyro = 1;

#ifndef LCD
  for (i=0; i<=NUM_ADC_Channels-1; i++) {
  	ADC_Data[curChannel] = 0;
  }
#endif

}

//--------------------------------------------------------------------------------------

void calcAngles(void) {
  double I;
  byte l;
  byte i;
  GyroChannelStruct * p;

  getTimeElapsed();

#ifdef LCD
//  LCD_CURSOR_LINE_1 (6);  WriteSNum ((int16) (GyroChannel[GYRO_VERT].raw_data[0]/GYRO_ADC_SAMPLES));
//  LCD_CURSOR_LINE_2 (6);  WriteSNum ((int16) (GyroChannel[GYRO_HORIZ].raw_data[0]/GYRO_ADC_SAMPLES));
#endif

  if (button)
	resetGyro = 1;

  for (i=GYRO_VERT; i<=GYRO_HORIZ; i++) {
	p = &(GyroChannel[i]);

  	// if reset buton is pressed, store current rate as a zero bias setting;
  	if (resetGyro) {
		p->raw_zero = p->raw_data[0];
     		for (l=0; l<=NUM_INTEGRATION_STEPS-1; l++) {
   			p->raw_data[l] = 0;
   		}
                p->angle = 0;
  	} else {

		// shift to signed format by zero bias substraction;
		p->raw_data[0] -= p->raw_zero;

#ifdef COARSE_READINGS
		if (p->raw_data[0] >= -GYRO_ADC_SAMPLES && p->raw_data[0] <= GYRO_ADC_SAMPLES)
			p->raw_data[0] = 0;
#endif
		// integrate by Runge-Kutta method;
		I = p->raw_data[1]+
			((double)(
			p->raw_data[0] + 2 * (p->raw_data[1]+p->raw_data[2]) + p->raw_data[3]
			))/6;

		// convert integration result to the real voltage value (mV);
		//	I = I/1024*VREF_VOLTAGE;

		// muliply integration result by sensitivity and delta time;
		p->angle += I*COEFF*((double)(timeElasped));	
  	}
  }
  resetGyro = 0;

//0123456789012345
//dxxx volt  ttttt
//axxx zero       

#ifdef LCD
//  LCD_CURSOR_LINE_1 (6);  WriteSNum ((int16) GyroChannel[GYRO_VERT].raw_data[0]);
//  LCD_CURSOR_LINE_2 (6);  WriteSNum ((int16) GyroChannel[GYRO_HORIZ].raw_data[0]);
  LCD_CURSOR_LINE_1 (0);  WriteSNum ((int16) GyroChannel[GYRO_HORIZ].angle);
  LCD_CURSOR_LINE_2 (0);  WriteSNum ((int16) GyroChannel[GYRO_VERT].angle);
  LCD_CURSOR_LINE_2 (11); WriteLNum (timeElasped);
#endif
}

//--------------------------------------------------------------------------------------

uint16 long_abs(int16 x) {
 if (x < 0)
   return (uint16) (-x);
 else
   return (uint16) x;
}

//--------------------------------------------------------------------------------------

// Limiting max speed of the value change;
uint16 limit(uint16 x, uint16 x_prev, byte max_speed) {
  int16 delta = x-x_prev;

  if (long_abs(delta) > max_speed) {
    return (delta < 0 ? x-max_speed : x+max_speed);
  } else {
    return x;
  }
}

//--------------------------------------------------------------------------------------

void translateAnglesToServo(void) {
  int16 v1;
  int16 h1;
  int ext;
  double angle;

  // Scale vertical movement v1 from 0;90 to -SERVO_SPAN;SERVO_SPAN;
  angle = GyroChannel[GYRO_VERT].angle;
  if (angle > MAX_ANGLE[GYRO_VERT])
    angle = MAX_ANGLE[GYRO_VERT];
  else 
  if (angle < 0)
    angle = 0;

  // Scale 0..45..90 -> -90..0..90
  angle = angle*2-MAX_ANGLE[GYRO_VERT];
  v1 = (int16) (angle/MAX_ANGLE[GYRO_VERT]*SERVO_SPAN);

  // Scale horizontal movement h1 from -90;90 to -SERVO_SPAN;SERVO_SPAN;
  angle = GyroChannel[GYRO_HORIZ].angle;
  if (angle > MAX_ANGLE[GYRO_HORIZ])
    angle = MAX_ANGLE[GYRO_HORIZ];
  else 
  if (angle < -MAX_ANGLE[GYRO_HORIZ])
    angle = -MAX_ANGLE[GYRO_HORIZ];

  h1 = (int16) (angle/MAX_ANGLE[GYRO_HORIZ]*SERVO_SPAN);

//--------------------------------------------------------------------------------------
// Changing servo throw by extensions;

#define v_ext (ADC_Data[iExtV])
#define h_ext (ADC_Data[iExtH])

// extension is about %50; radius of 500 around neutral becomes 750; 
// servo will be driven by impulses in range of 750..2250 ms (ext. 0% is default 1000..2000 ms);

  ext = ( long_abs(v1) * ((v_ext+1)/16) ) / 128; // v_ext=0..1023;
  if (v1 < 0) 
    v1 -= (int16) ext;
  else
    v1 += (int16) ext;

  ext = ( long_abs(h1) * ((h_ext+1)/16) ) / 128; // v_ext=0..1023;
  if (h1 < 0) 
    h1 -= (int16) ext;
  else
    h1 += (int16) ext;

//--------------------------------------------------------------------------------------
// Changing servo neutral point by trimming;

#define v_trim (ADC_Data[iTrimV])
#define h_trim (ADC_Data[iTrimH])

// trimming is up to about 50%; 1024/2 ADC value means 0%; 0 means -50%; 1023 means +50%;
// Up to about 256 ms added;

  v1 += ((v_trim+1)/2)-(1024/4);
  h1 += ((h_trim+1)/2)-(1024/4);

//--------------------------------------------------------------------------------------
// performing reversing;

  if (Horiz_Rev)
    h1 = -h1;

  if (Vert_Rev)
    v1 = -v1;

//--------------------------------------------------------------------------------------
// final shifting;

  v = (uint16) (v1+SERVO_NEUTRAL+(SHIFT_NEUTRAL)); 
  h = (uint16) (h1+SERVO_NEUTRAL+(SHIFT_NEUTRAL));
}

//--------------------------------------------------------------------------------------

// Input: v, h;
// Output: v, h;
// Changes: ServoData_H, ServoData_V;

void doFiltering(void) {
  byte i;
  uint16 unfiltered_v;
  uint16 unfiltered_h;

// averaging filter (sliding window);

  // sum data from NUM_AVG_SAMPLES-1 previous frames and add v, h;
  // shift ServoData arrays towards the begining;
  // preserve unfiltered values for adding to averaging array;

  // Divide by 2 to keep sum in the range, eliminate jitter too.
  v /= 2;
  h /= 2;

  unfiltered_v = v;
  unfiltered_h = h;

  // add first items before, because they will be shifted out
  h += ServoData_H[0];  
  v += ServoData_V[0];  

  for (i=0; i < NUM_AVG_SAMPLES-2; i++) { // 0,1,2
    uint16 t = ServoData_H[i+1];
    ServoData_H[i] = t;
    h += t;
    t = ServoData_V[i+1];
    ServoData_V[i] = t;
    v += t;
  }
  v /= (NUM_AVG_SAMPLES/2); 
  h /= (NUM_AVG_SAMPLES/2);

  // round by 16 to eliminate tiny movements;
//  v.low8 &= 0xFFF0;
//  h.low8 &= 0xFFF0;

  // put latest values in to averaging array;
  ServoData_H[NUM_AVG_SAMPLES-2] = unfiltered_h;
  ServoData_V[NUM_AVG_SAMPLES-2] = unfiltered_v;
}

//--------------------------------------------------------------------------------------
// speed limiting;

void speedLimiting(void) {
  v = limit(v, ServoData_V[NUM_AVG_SAMPLES-2], MAX_VERT_SERVO_SPEED);
  h = limit(h, ServoData_H[NUM_AVG_SAMPLES-2], MAX_HORIZ_SERVO_SPEED);
}
                                                                                                      
void process(void) {

  readChannels();

  calcAngles();

  translateAnglesToServo();

#ifdef SPEED_LIMITING
  speedLimiting();
#endif

#ifdef FILTERING
  doFiltering();
#endif

//--------------------------------------------------------------------------------------
// update servo movements;

  GIE = 0; Servo[1] = v; GIE = 1;
  GIE = 0; Servo[0] = h; GIE = 1;
  if (button) {
    GIE = 0; Servo[2] = SERVO_MIN; GIE = 1;
  } else {
    GIE = 0; Servo[2] = SERVO_MAX; GIE = 1;
  }  

//0123456789012345
//xxxx yyyy   ffff
//ssss ssss   ffff

#ifdef LCD
    LCD_CURSOR_LINE_1 (6);  WriteNum (Servo[0]);
    LCD_CURSOR_LINE_2 (6);  WriteNum (Servo[1]);
#ifdef FRAMES_COUNT
    LCD_CURSOR_LINE_1 (11);  WriteNum (FramesCount);
//    LCD_CURSOR_LINE_2 (12);  WriteNum (BrokenFramesCount);  
#endif
//  LCD_CURSOR_LINE_1 (10);  Write1Digit (_StartCh);  
//  LCD_CURSOR_LINE_2 (12);  WriteNum (widthHoriz);  
#endif
}

//--------------------------------------------------------------------------------------

void InitServo(void) {
  // init channels;
  byte i;
  for (i=0; i<NUM_SERVO; i++)
    Servo[i]= SERVO_NEUTRAL;
  for (i=0; i<NUM_AVG_SAMPLES-1; i++) {
    ServoData_V[i]= SERVO_NEUTRAL/2; // Divide by 2 to follow averaging code convention;
    ServoData_H[i]= SERVO_NEUTRAL/2;
  }
#ifdef FRAMES_COUNT
  BrokenFramesCount = 0;
  FramesCount = 0;
#endif
  CurrentServo = 0;
  ServoStart = 0;
  SERVO_OUT = !SERVO_HIGH;
}

//--------------------------------------------------------------------------------------

void StartWaitForGapEx(void) {
  WaitForGap = 1; 
  // find 3 slot silence at least in TX sending;
  T1CON = 0b00000000; // Stop timer;
  {
           uint16 t;
           ((WORD*)(&t))->low = TMR1L; 
           ((WORD*)(&t))->high = TMR1H; 
           t = 65536-t; 
           delayCounter += (TMR1_Value-t)/2;
           numIncrements++;

  }
  TMR1H = ((65536-SERVO_GAP*2) >> 8) & 0xFF; 
  TMR1L = (65536-SERVO_GAP*2) & 0xFF; // start counting;
  T1CON = 0b00000001; // Resume counting;
  TMR1_Value = SERVO_GAP*2;
}

void StartWaitForGap(void) {
  WaitForGap = 1; 
  // find 3 slot silence at least in TX sending;
  T1CON = 0b00000000; // Stop timer;
  TMR1H = ((65536-SERVO_GAP*2) >> 8) & 0xFF; 
  TMR1L = (65536-SERVO_GAP*2) & 0xFF; // start counting;
  T1CON = 0b00000001; // Resume counting;
  TMR1_Value = SERVO_GAP*2;
}


//--------------------------------------------------------------------------------------

void main(void) {

  TRISA = TrisA;
  TRISC = TrisC;
  OPTION = 0b11001110; // no pull-up, int/RA2 on 0->1; Prescalder 64 -> ~1 sec. to WDT;
  OSCCON = 0b01110111; // 8 MHz;
  PORTA = 0; 
  PORTC = 0;
  button = 0;

  CMCON0 = 0b00000111; // comparator is turned off;

  ANSEL = (1 << ADChannel_GYRO_VERT) | (1 << ADChannel_GYRO_HORIZ) |
#ifdef LCD
          0;
#else
          (1 << ADChannel_TrimV) | (1 << ADChannel_TrimH) | 
          (1 << ADChannel_ExtV ) | (1 << ADChannel_ExtH );
#endif


  ADCON1 = 0b01010000;      //  Select the Clock as Fosc/16

  ADIE = 0;

  ADCON0 = 0b11000001;     //  Turn on the ADC
                              //   Bit 7 - Right Justified Sample
                              //   Bit 6 - Use VRef
                              //   Bit 4:2 - Channel 0
                              //   Bit 1 - Do not Start
                              //   Bit 0 - Turn on ADC
#ifdef LCD
  LCD_Init();          
#endif
   
  InitServo();
  initADC();

  Delay10ms(30);
  Delay10ms(1);
  Delay10ms(2);

  StartWaitForGap();

  // Init Timer1;
  TMR1IF = 0;           
  TMR1IE = 1;

  // Init TMR0;
  T0IE = 0;

  // // Enable SERVO_IN edge interrupt;
  INTF = 0;
  INTE = 1; 

  // Turn on all interrupt levels;
  PIE1 = 1; 
  PEIE = 1;
  GIE = 1; // Enable global interrupts;

  getTimeElapsed();

  while (1) {
    CLRWDT(); 
    process();
  };
}

